"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var OrmUtils_1 = require("../../util/OrmUtils");
/**
 * Transforms raw sql results returned from the database into entity object.
 * Entity is constructed based on its entity metadata.
 */
var RawSqlResultsToEntityTransformer = (function () {
    // -------------------------------------------------------------------------
    // Constructor
    // -------------------------------------------------------------------------
    function RawSqlResultsToEntityTransformer(driver, expressionMap) {
        this.driver = driver;
        this.expressionMap = expressionMap;
    }
    // -------------------------------------------------------------------------
    // Public Methods
    // -------------------------------------------------------------------------
    RawSqlResultsToEntityTransformer.prototype.transform = function (rawSqlResults, rawRelationIdResults) {
        this.rawRelationIdResults = rawRelationIdResults;
        return this.groupAndTransform(rawSqlResults, this.expressionMap.mainAlias.name, this.expressionMap.mainAlias.metadata);
    };
    // -------------------------------------------------------------------------
    // Private Methods
    // -------------------------------------------------------------------------
    /**
     * Since db returns a duplicated rows of the data where accuracies of the same object can be duplicated
     * we need to group our result and we must have some unique id (primary key in our case)
     */
    RawSqlResultsToEntityTransformer.prototype.groupAndTransform = function (rawSqlResults, alias, metadata) {
        var _this = this;
        var groupedResults = OrmUtils_1.OrmUtils.groupBy(rawSqlResults, function (result) {
            return metadata.primaryColumnsWithParentIdColumns.map(function (column) { return result[alias + "_" + column.fullName]; }).join("_"); // todo: check it
        });
        return groupedResults
            .map(function (group) { return _this.transformIntoSingleResult(group.items, alias, metadata); })
            .filter(function (res) { return !!res; });
    };
    /**
     * Transforms set of data results into single entity.
     */
    RawSqlResultsToEntityTransformer.prototype.transformIntoSingleResult = function (rawSqlResults, alias, metadata) {
        var _this = this;
        var entity = metadata.create();
        var hasData = false;
        this.rawRelationIdResults
            .filter(function (rawRelationIdResult) { return rawRelationIdResult.relationIdAttribute.parentAlias === alias; })
            .forEach(function (rawRelationIdResult) {
            var relation = rawRelationIdResult.relationIdAttribute.relation;
            var referenceColumnName;
            if (relation.isManyToOne || relation.isOneToOneOwner) {
                referenceColumnName = "";
            }
            else if (relation.isOneToMany || (relation.isOneToOne && !relation.isOwning)) {
                referenceColumnName = relation.referencedColumn.fullName;
            }
            else {
                referenceColumnName = relation.isOwning ? relation.joinTable.referencedColumn.fullName : relation.inverseRelation.joinTable.referencedColumn.fullName;
            }
            var referenceColumnValue = rawSqlResults[0][alias + "_" + referenceColumnName]; // we use zero index since its grouped data // todo: selection with alias for entity columns wont work
            if (referenceColumnValue !== undefined && referenceColumnValue !== null) {
                rawRelationIdResult.results
                    .filter(function (result) { return result["parentId"] === referenceColumnValue; })
                    .forEach(function (result) {
                    if (!entity[rawRelationIdResult.relationIdAttribute.mapToPropertyPropertyName])
                        entity[rawRelationIdResult.relationIdAttribute.mapToPropertyPropertyName] = [];
                    if (relation.isOneToOne && !relation.isOwning) {
                        entity[rawRelationIdResult.relationIdAttribute.mapToPropertyPropertyName] = result["id"];
                    }
                    else {
                        entity[rawRelationIdResult.relationIdAttribute.mapToPropertyPropertyName].push(result["id"]);
                    }
                    hasData = true;
                });
            }
        });
        // add special columns that contains relation ids
        if (this.expressionMap.enableRelationIdValues) {
            metadata.columns
                .filter(function (column) { return !!column.relationMetadata; })
                .forEach(function (column) {
                var valueInObject = rawSqlResults[0][alias + "_" + column.fullName]; // we use zero index since its grouped data
                if (valueInObject !== undefined && valueInObject !== null && column.propertyName) {
                    entity[column.propertyName] = _this.driver.prepareHydratedValue(valueInObject, column);
                    hasData = true;
                }
            });
        }
        this.expressionMap.joinAttributes.forEach(function (join) {
            if (!join.mapToProperty)
                return;
            var _a = join.mapToProperty.split("."), parentName = _a[0], propertyName = _a[1];
            if (parentName !== alias /*  || join.parentAlias|| !join.destinationSelection.target*/)
                return;
            if (!join.metadata)
                return;
            var relatedEntities = _this.groupAndTransform(rawSqlResults, join.alias, join.metadata);
            var isResultArray = join.isMappingMany;
            var result = !isResultArray ? relatedEntities[0] : relatedEntities;
            if (result && (!isResultArray || result.length > 0)) {
                entity[propertyName] = result;
                hasData = true;
            }
        });
        /*

        this.joinAttributes
            .filter(join => join.type === "relationId")
            .forEach(relationIdAttr => {
                if (relationIdAttr.mapToPropertyParentAlias !== alias)
                    return;

                if (relationIdAttr.relation!.isManyToMany) {
                    entity[relationIdAttr.mapToPropertyPropertyName] = rawSqlResults.map(raw => {
                        const [firstColumn, secondColumn] = relationIdAttr.relation!.junctionEntityMetadata.columns;
                        const column = relationIdAttr.relation!.isOwning ? secondColumn : firstColumn;
                        return raw[relationIdAttr.junctionAlias + "_" + column.fullName];
                    });
                    hasData = true;

                } else if (relationIdAttr.relation!.isManyToOne || relationIdAttr.relation!.isOneToOneOwner) {
                    const relationIds = rawSqlResults.map(raw => {
                        return raw[relationIdAttr.parentAlias + "_" + relationIdAttr.relation!.name];
                    });
                    if (relationIds.length) {
                        entity[relationIdAttr.mapToPropertyPropertyName] = relationIds[0];
                        hasData = true;
                    }
                }

                /!*const relatedEntities = this.groupAndTransform(rawSqlResults, relationIdAttr.junctionAlias, relationIdAttr.junctionMetadata); // todo: find out what metadata should be passed
                console.log("relatedEntities", relatedEntities);
                const result = relatedEntities[0];

                if (result && result.length > 0) {
                    entity[relationIdAttr.mapToPropertyPropertyName] = result;
                    hasData = true;
                }*!/
            });*/
        // get value from columns selections and put them into object
        metadata.columnsWithoutEmbeddeds.forEach(function (column) {
            var columnName = column.fullName;
            var valueInObject = rawSqlResults[0][alias + "_" + columnName]; // we use zero index since its grouped data
            if (valueInObject !== undefined && valueInObject !== null && column.propertyName && !column.isVirtual && !column.isParentId && !column.isDiscriminator) {
                var value = _this.driver.prepareHydratedValue(valueInObject, column);
                // if (column.isInEmbedded) {
                //     if (!entity[column.embeddedProperty])
                //         entity[column.embeddedProperty] = column.embeddedMetadata.create();
                //
                //     entity[column.embeddedProperty][column.propertyName] = value;
                // } else {
                entity[column.propertyName] = value;
                // }
                hasData = true;
            }
        });
        var addEmbeddedValuesRecursively = function (entity, embeddeds) {
            embeddeds.forEach(function (embedded) {
                embedded.columns.forEach(function (column) {
                    var value = rawSqlResults[0][alias + "_" + column.fullName];
                    if (!value)
                        return;
                    if (!entity[embedded.propertyName])
                        entity[embedded.propertyName] = embedded.create();
                    entity[embedded.propertyName][column.propertyName] = value;
                    hasData = true;
                });
                addEmbeddedValuesRecursively(entity[embedded.propertyName], embedded.embeddeds);
            });
        };
        addEmbeddedValuesRecursively(entity, metadata.embeddeds);
        // add parent tables metadata
        // console.log(rawSqlResults);
        // todo: duplication
        if (metadata.parentEntityMetadata) {
            metadata.parentEntityMetadata.columnsWithoutEmbeddeds.forEach(function (column) {
                var columnName = column.fullName;
                var valueInObject = rawSqlResults[0]["parentIdColumn_" + metadata.parentEntityMetadata.table.name + "_" + columnName]; // we use zero index since its grouped data
                if (valueInObject !== undefined && valueInObject !== null && column.propertyName && !column.isVirtual && !column.isParentId && !column.isDiscriminator) {
                    var value = _this.driver.prepareHydratedValue(valueInObject, column);
                    // if (column.isInEmbedded) {
                    //     if (!entity[column.embeddedProperty])
                    //         entity[column.embeddedProperty] = column.embeddedMetadata.create();
                    //
                    //     entity[column.embeddedProperty][column.propertyName] = value;
                    // } else {
                    entity[column.propertyName] = value;
                    // }
                    hasData = true;
                }
            });
            addEmbeddedValuesRecursively(entity, metadata.parentEntityMetadata.embeddeds);
        }
        // if relation is loaded then go into it recursively and transform its values too
        metadata.relations.forEach(function (relation) {
            var join = _this.expressionMap.joinAttributes.find(function (join) { return join.parentAlias === alias && join.relationProperty === relation.propertyName; });
            if (join && join.metadata) {
                var relatedEntities = _this.groupAndTransform(rawSqlResults, join.alias, join.metadata);
                var isResultArray = relation.isManyToMany || relation.isOneToMany;
                var result = !isResultArray ? relatedEntities[0] : relatedEntities;
                if (result) {
                    var propertyName = relation.propertyName;
                    if (join.mapToProperty) {
                        propertyName = join.mapToProperty.split(".")[1];
                    }
                    if (relation.isLazy) {
                        entity["__" + propertyName + "__"] = result;
                    }
                    else {
                        entity[propertyName] = result;
                    }
                    if (!isResultArray || result.length > 0)
                        hasData = true;
                }
            }
            // if relation has id field then relation id/ids to that field.
            if (relation.isManyToMany) {
                if (join) {
                    var ids_1 = [];
                    var joinRelationId = _this.expressionMap.joinAttributes.find(function (join) { return join.junctionAlias === join.alias; });
                    if (relation.idField || joinRelationId) {
                        var propertyName = "";
                        if (joinRelationId && joinRelationId.mapToProperty) {
                            propertyName = joinRelationId.mapToProperty.split(".")[1];
                        }
                        else {
                            propertyName = relation.idField;
                        }
                        var junctionMetadata = relation.junctionEntityMetadata;
                        var columnName_1 = relation.isOwning ? junctionMetadata.columns[1].fullName : junctionMetadata.columns[0].fullName;
                        rawSqlResults.forEach(function (results) {
                            if (join) {
                                var resultsKey = join.alias + "_" + columnName_1;
                                var value = _this.driver.prepareHydratedValue(results[resultsKey], relation.referencedColumn);
                                if (value !== undefined && value !== null)
                                    ids_1.push(value);
                            }
                        });
                        if (ids_1 && ids_1.length)
                            entity[propertyName] = ids_1;
                    }
                }
            }
            else if (relation.idField) {
                var relationName = relation.name;
                entity[relation.idField] = _this.driver.prepareHydratedValue(rawSqlResults[0][alias + "_" + relationName], relation.referencedColumn);
            }
            // if relation counter
            _this.expressionMap.relationCountAttributes.forEach(function (joinMeta) {
                if (join && joinMeta.junctionAlias === join.parentAlias) {
                    // console.log("relation count was found for relation: ", relation);
                    // joinMeta.entity = entity;
                    joinMeta.entities.push({ entity: entity, metadata: metadata });
                    // console.log(joinMeta);
                    // console.log("---------------------");
                }
            });
        });
        return hasData ? entity : null;
    };
    return RawSqlResultsToEntityTransformer;
}());
exports.RawSqlResultsToEntityTransformer = RawSqlResultsToEntityTransformer;

//# sourceMappingURL=RawSqlResultsToEntityTransformer.js.map
